import { jsx as _jsx, Fragment as _Fragment, jsxs as _jsxs } from "react/jsx-runtime";
import { useCallback, useEffect, useMemo, useRef } from 'react';
import { ResponsiveChart } from 'react-d3-utils';
import { assert } from 'react-science/ui';
import BrushXY, { BRUSH_TYPE } from '../1d-2d/tools/BrushXY';
import CrossLinePointer from '../1d-2d/tools/CrossLinePointer';
import { BrushTracker, } from '../EventsTrackers/BrushTracker';
import { MouseTracker } from '../EventsTrackers/MouseTracker';
import { useChartData } from '../context/ChartContext';
import { useDispatch } from '../context/DispatchContext';
import { useMapKeyModifiers } from '../context/KeyModifierContext';
import Spinner from '../loader/Spinner';
import { options } from '../toolbar/ToolTypes';
import { PhaseTraces } from './1d-tracer/phase-correction-traces';
import Chart2D from './Chart2D';
import FooterBanner from './FooterBanner';
import SlicingView from './SlicingView';
import PivotIndicator from './tools/PivotIndicator';
import XYLabelPointer from './tools/XYLabelPointer';
import { get2DDimensionLayout, getLayoutID } from './utilities/DimensionLayout';
import { get2DXScale, get2DYScale } from './utilities/scale';
function Viewer2D({ emptyText = undefined }) {
    const state = useChartData();
    const { toolOptions: { selectedTool }, isLoading, data, margin, view: { spectra: { activeSpectra, activeTab }, }, } = state;
    const dispatch = useDispatch();
    const brushStartRef = useRef(null);
    const { getModifiersKey, primaryKeyIdentifier } = useMapKeyModifiers();
    const spectrumData = useMemo(() => {
        const nuclei = activeTab.split(',');
        return nuclei
            .map((nucleus) => {
            const spectra = activeSpectra[nucleus];
            if (spectra?.length === 1) {
                const id = spectra[0].id;
                const spectrum = data.find((datum) => datum.id === id && !datum.info.isFid);
                assert(spectrum, `Spectrum with id ${id} not found`);
                return spectrum;
            }
            return null;
        })
            .filter((d) => d !== null);
    }, [activeTab, data, activeSpectra]);
    const DIMENSION = get2DDimensionLayout(state);
    function handelBrush(brushData) {
        const { startX: startXInPixel, endX: endXInPixel, startY: startYInPixel, endY: endYInPixel, mouseButton, } = brushData;
        if (mouseButton === 'secondary') {
            const scaleX = get2DXScale(state);
            const scaleY = get2DYScale(state);
            if (!brushStartRef.current) {
                const x = scaleX.invert(startXInPixel);
                const y = scaleY.invert(startYInPixel);
                brushStartRef.current = { x, y };
            }
            const { x, y } = brushStartRef.current;
            const shiftX = scaleX.invert(endXInPixel) - x;
            const shiftY = scaleY.invert(endYInPixel) - y;
            dispatch({ type: 'MOVE', payload: { shiftX, shiftY } });
        }
    }
    const handelBrushEnd = useCallback((brushData) => {
        //reset the brush start
        brushStartRef.current = null;
        const modifierKey = getModifiersKey(brushData);
        let executeDefaultAction = false;
        if (brushData.mouseButton === 'main') {
            const trackID = getLayoutID(DIMENSION, brushData);
            if (trackID) {
                switch (modifierKey) {
                    case primaryKeyIdentifier: {
                        switch (selectedTool) {
                            case options.zoom.id: {
                                executeDefaultAction = true;
                                break;
                            }
                            case options.zonePicking.id: {
                                dispatch({ type: 'ADD_2D_ZONE', payload: brushData });
                                break;
                            }
                            default:
                                break;
                        }
                        break;
                    }
                    default: {
                        executeDefaultAction = true;
                        break;
                    }
                }
                const isNotDistanceMeasurementTool = selectedTool !== 'zoom' ||
                    (selectedTool === 'zoom' && !brushData.shiftKey);
                if (executeDefaultAction &&
                    selectedTool != null &&
                    isNotDistanceMeasurementTool) {
                    return dispatch({
                        type: 'BRUSH_END',
                        payload: {
                            ...brushData,
                            trackID: getLayoutID(DIMENSION, brushData),
                        },
                    });
                }
            }
        }
    }, [getModifiersKey, DIMENSION, selectedTool, primaryKeyIdentifier, dispatch]);
    const handelOnDoubleClick = useCallback((e) => {
        const { x: startX, y: startY } = e;
        const trackID = getLayoutID(DIMENSION, { startX, startY });
        if (trackID) {
            dispatch({ type: 'FULL_ZOOM_OUT', payload: { trackID } });
        }
    }, [DIMENSION, dispatch]);
    const handleZoom = (event) => {
        const { x: startX, y: startY, shiftKey } = event;
        const trackID = getLayoutID(DIMENSION, { startX, startY });
        if (trackID) {
            if (trackID !== 'CENTER_2D' ||
                (selectedTool === 'phaseCorrectionTwoDimensions' && !shiftKey)) {
                dispatch({ type: 'SET_ZOOM', payload: { event, trackID } });
            }
            else {
                dispatch({ type: 'SET_2D_LEVEL', payload: event });
            }
        }
    };
    const mouseClick = useCallback((event) => {
        const { x, y, shiftKey } = event;
        if (shiftKey) {
            switch (selectedTool) {
                case 'phaseCorrectionTwoDimensions':
                    dispatch({
                        type: 'SET_TWO_DIMENSION_PIVOT_POINT',
                        payload: { x, y },
                    });
                    break;
                default:
                    break;
            }
        }
        else {
            switch (selectedTool) {
                case 'phaseCorrectionTwoDimensions':
                    dispatch({ type: 'ADD_PHASE_CORRECTION_TRACE', payload: { x, y } });
                    break;
                default:
                    break;
            }
        }
    }, [selectedTool, dispatch]);
    return (_jsx(ResponsiveChart, { children: ({ width, height }) => (_jsxs(ViewerResponsiveWrapper, { width: width, height: height, children: [_jsx(Spinner, { isLoading: isLoading, emptyText: emptyText }), data && data.length > 0 && (_jsx(BrushTracker, { onBrush: handelBrush, onBrushEnd: handelBrushEnd, onDoubleClick: handelOnDoubleClick, onClick: mouseClick, onZoom: handleZoom, style: {
                        width: '100%',
                        height: '100%',
                        margin: 'auto',
                        position: 'relative',
                        overflow: 'hidden',
                    }, children: _jsxs(MouseTracker, { style: { width: '100%', height: `100%`, position: 'relative' }, children: [selectedTool && selectedTool === options.slicing.id && (_jsx(SlicingView, {})), selectedTool &&
                                selectedTool === options.phaseCorrectionTwoDimensions.id && (_jsx(PhaseTraces, {})), _jsx(CrossLinePointer, {}), _jsx(XYLabelPointer, { data1D: spectrumData, layout: DIMENSION }), _jsx(BrushXY, { brushType: BRUSH_TYPE.XY, dimensionBorder: DIMENSION.CENTER_2D }), _jsxs(_Fragment, { children: [spectrumData[0] && (_jsx(BrushXY, { brushType: BRUSH_TYPE.X, dimensionBorder: DIMENSION.TOP_1D, height: margin.top })), spectrumData[1] && (_jsx(BrushXY, { brushType: BRUSH_TYPE.Y, dimensionBorder: DIMENSION.LEFT_1D, width: margin.left }))] }), _jsx(PivotIndicator, {}), _jsx(FooterBanner, { data1D: spectrumData, layout: DIMENSION }), _jsx(Chart2D, { spectra: spectrumData })] }) }))] })) }));
}
export function ViewerResponsiveWrapper(props) {
    const dispatch = useDispatch();
    const { width, height, children } = props;
    useEffect(() => {
        dispatch({ type: 'SET_DIMENSIONS', payload: { width, height } });
    }, [width, height, dispatch]);
    return children;
}
export default Viewer2D;
