import { jsx as _jsx, jsxs as _jsxs } from "@emotion/react/jsx-runtime";
/** @jsxImportSource @emotion/react */
import { css } from '@emotion/react';
import { SvgNmrFt, SvgNmrPeaks, SvgNmrPeaksTopLabels } from 'cheminfo-font';
import SvgPeaks from 'cheminfo-font/lib-react-cjs/lib-react-tsx/nmr/Peaks';
import { useCallback, useMemo, useState, useRef, memo } from 'react';
import { FaThinkPeaks } from 'react-icons/fa';
import isInRange from '../../../data/utilities/isInRange';
import { useChartData } from '../../context/ChartContext';
import { useDispatch } from '../../context/DispatchContext';
import { usePreferences } from '../../context/PreferencesContext';
import { useAlert } from '../../elements/popup/Alert';
import { useModal } from '../../elements/popup/Modal';
import { useActiveSpectrumPeaksViewState } from '../../hooks/useActiveSpectrumPeaksViewState';
import useCheckExperimentalFeature from '../../hooks/useCheckExperimentalFeature';
import { useFormatNumberByNucleus } from '../../hooks/useFormatNumberByNucleus';
import useSpectrum from '../../hooks/useSpectrum';
import { booleanToString } from '../../utility/booleanToString';
import { tablePanelStyle } from '../extra/BasicPanelStyle';
import DefaultPanelHeader from '../header/DefaultPanelHeader';
import PreferencesHeader from '../header/PreferencesHeader';
import PeaksPreferences from './PeaksPreferences';
import PeaksTable from './PeaksTable';
function PeaksPanelInner({ peaks, info, xDomain, activeTab, peaksViewState, }) {
    const [filterIsActive, setFilterIsActive] = useState(false);
    const [isFlipped, setFlipStatus] = useState(false);
    const format = useFormatNumberByNucleus(info.nucleus);
    const dispatch = useDispatch();
    const modal = useModal();
    const alert = useAlert();
    const isExperimental = useCheckExperimentalFeature();
    const settingRef = useRef();
    const yesHandler = useCallback(() => {
        dispatch({ type: 'DELETE_PEAK', payload: {} });
    }, [dispatch]);
    const handleDeleteAll = useCallback(() => {
        modal.showConfirmDialog({
            message: 'All records will be deleted, Are You sure?',
            buttons: [{ text: 'Yes', handler: yesHandler }, { text: 'No' }],
        });
    }, [modal, yesHandler]);
    const settingsPanelHandler = useCallback(() => {
        setFlipStatus(!isFlipped);
    }, [isFlipped]);
    const saveSettingHandler = useCallback(() => {
        settingRef.current.saveSetting();
        setFlipStatus(false);
    }, []);
    const handleOnFilter = useCallback(() => {
        setFilterIsActive(!filterIsActive);
    }, [filterIsActive]);
    const filteredPeaks = useMemo(() => {
        if (peaks?.values) {
            const [from, to] = xDomain;
            const _peaks = filterIsActive
                ? peaks.values.filter((peak) => isInRange(peak.x, { from, to }))
                : peaks.values;
            return _peaks
                .map((peak) => {
                const { x, y, width, ...peakProperties } = peak;
                const value = Number(format(x));
                return {
                    ...peakProperties,
                    x,
                    xHz: info?.originFrequency && value * info.originFrequency,
                    y,
                    width,
                    isConstantlyHighlighted: isInRange(value, { from, to }),
                };
            })
                .sort((prev, next) => prev.x - next.x);
        }
        return [];
    }, [filterIsActive, format, info, peaks, xDomain]);
    const optimizePeaksHandler = () => {
        const [from, to] = xDomain;
        const filterPeaks = peaks.values.filter((peak) => isInRange(peak.x, { from, to }));
        if (filterPeaks.length <= 4) {
            dispatch({ type: 'OPTIMIZE_PEAKS', payload: { peaks: filterPeaks } });
        }
        else {
            alert.error('optimization can be done on no more than 4 peaks');
        }
    };
    function toggleViewProperty(key) {
        dispatch({ type: 'TOGGLE_PEAKS_VIEW_PROPERTY', payload: { key } });
    }
    function toggleDisplayingMode() {
        dispatch({ type: 'TOGGLE_PEAKS_DISPLAYING_MODE' });
    }
    const total = peaks?.values?.length || 0;
    const disabled = !peaks?.values || peaks.values.length === 0;
    const { showPeaks, displayingMode, showPeaksShapes, showPeaksSum } = peaksViewState;
    const leftButtons = [
        {
            disabled,
            icon: _jsx(SvgNmrPeaks, {}),
            title: `${booleanToString(!showPeaks)} peaks`,
            onClick: () => toggleViewProperty('showPeaks'),
            active: showPeaks,
        },
        {
            disabled,
            icon: _jsx(SvgNmrPeaksTopLabels, {}),
            title: `${displayingMode === 'spread' ? 'Single' : 'Spread'} mode`,
            onClick: toggleDisplayingMode,
            active: displayingMode === 'spread',
        },
    ];
    if (isExperimental) {
        leftButtons.unshift({
            disabled,
            icon: _jsx(SvgPeaks, {}),
            title: `${booleanToString(!showPeaksShapes)} peaks shapes`,
            onClick: () => toggleViewProperty('showPeaksShapes'),
            active: showPeaksShapes,
        }, {
            disabled,
            icon: _jsx(SvgNmrFt, {}),
            title: `${booleanToString(!showPeaksSum)} peaks sum`,
            onClick: () => toggleViewProperty('showPeaksSum'),
            active: showPeaksSum,
        }, {
            icon: _jsx(FaThinkPeaks, {}),
            title: 'Optimize peaks',
            onClick: optimizePeaksHandler,
        });
    }
    return (_jsxs("div", { css: [
            tablePanelStyle,
            isFlipped &&
                css `
            .table-container {
              table,
              th {
                position: relative !important;
              }
            }
          `,
        ], children: [!isFlipped && (_jsx(DefaultPanelHeader, { total: total, counter: filteredPeaks?.length, onDelete: handleDeleteAll, deleteToolTip: "Delete All Peaks", onFilter: handleOnFilter, filterToolTip: filterIsActive ? 'Show all peaks' : 'Hide peaks out of view', onSettingClick: settingsPanelHandler, leftButtons: leftButtons })), isFlipped && (_jsx(PreferencesHeader, { onSave: saveSettingHandler, onClose: settingsPanelHandler })), _jsx("div", { className: "inner-container", children: !isFlipped ? (_jsx(PeaksTable, { data: filteredPeaks, info: info, activeTab: activeTab })) : (_jsx(PeaksPreferences, { ref: settingRef })) })] }));
}
const MemoizedPeaksPanel = memo(PeaksPanelInner);
const emptyData = { peaks: null, info: {} };
export default function PeaksPanel() {
    const { xDomain, view: { spectra: { activeTab }, }, } = useChartData();
    const { peaks, info } = useSpectrum(emptyData);
    const preferences = usePreferences();
    const peaksViewState = useActiveSpectrumPeaksViewState();
    return (_jsx(MemoizedPeaksPanel, { peaks, info, xDomain, activeTab, preferences, peaksViewState }));
}
