import { jsx as _jsx } from "react/jsx-runtime";
import lodashGet from 'lodash/get';
import { useCallback, useMemo, memo } from 'react';
import { FaEdit, FaRegTrashAlt } from 'react-icons/fa';
import { useDispatch } from '../../context/DispatchContext';
import EditableColumn from '../../elements/EditableColumn';
import ReactTable from '../../elements/ReactTable/ReactTable';
import addCustomColumn, { createActionColumn, } from '../../elements/ReactTable/utility/addCustomColumn';
import { useModal } from '../../elements/popup/Modal';
import { positions, transitions } from '../../elements/popup/options';
import { usePanelPreferences } from '../../hooks/usePanelPreferences';
import EditPeakShapeModal from '../../modal/EditPeakShapeModal';
import { formatNumber } from '../../utility/formatNumber';
import NoDataForFid from '../extra/placeholder/NoDataForFid';
import NoTableData from '../extra/placeholder/NoTableData';
function handleActiveRow(row) {
    return row.original.isConstantlyHighlighted;
}
function PeaksTable({ activeTab, data, info }) {
    const dispatch = useDispatch();
    const modal = useModal();
    const peaksPreferences = usePanelPreferences('peaks', activeTab);
    const deletePeakHandler = useCallback((row) => {
        const params = row.original;
        dispatch({
            type: 'DELETE_PEAK',
            payload: { id: params.id },
        });
    }, [dispatch]);
    const editPeakHandler = useCallback((row) => {
        modal.show(_jsx(EditPeakShapeModal, { peak: row.original, peaksPreferences: peaksPreferences }), {
            position: positions.MIDDLE_RIGHT,
            transition: transitions.SCALE,
            isBackgroundBlur: false,
        });
    }, [modal, peaksPreferences]);
    const saveDeltaPPMRefsHandler = useCallback((event, row) => {
        const shift = Number.parseFloat(event.target.value) - Number.parseFloat(row.x);
        dispatch({ type: 'SHIFT_SPECTRUM', payload: { shift } });
    }, [dispatch]);
    const COLUMNS = useMemo(() => [
        {
            showWhen: 'showSerialNumber',
            index: 1,
            Header: '#',
            accessor: (_, index) => index + 1,
            style: { width: '1%', maxWidth: '40px', minWidth: '40px' },
        },
        {
            showWhen: 'deltaPPM.show',
            index: 3,
            Header: 'δ (ppm)',
            accessor: 'x',
            Cell: ({ row }) => (_jsx(EditableColumn, { value: formatNumber(row.original.x, peaksPreferences.deltaPPM.format), onSave: (event) => saveDeltaPPMRefsHandler(event, row.original), type: "number", validate: (val) => val !== '' })),
        },
        {
            showWhen: 'deltaHz.show',
            index: 4,
            Header: 'δ (Hz)',
            accessor: 'xHz',
            Cell: ({ row }) => formatNumber(row.original.xHz, peaksPreferences.deltaHz.format),
        },
        {
            showWhen: 'intensity.show',
            index: 5,
            Header: 'Intensity',
            style: { maxWidth: '80px' },
            accessor: 'y',
            Cell: ({ row }) => formatNumber(row.original.y, peaksPreferences.intensity.format),
        },
        {
            showWhen: 'peakWidth.show',
            index: 6,
            Header: 'Width (Hz)',
            accessor: 'width',
            Cell: ({ row }) => formatNumber(row.original.width, peaksPreferences.peakWidth.format),
        },
        {
            showWhen: 'showKind',
            index: 7,
            Header: 'Kind',
            accessor: (row) => row.shape?.kind || '',
        },
        {
            showWhen: 'fwhm.show',
            index: 8,
            Header: 'fwhm',
            accessor: (row) => row?.shape?.fwhm || '',
            Cell: ({ row }) => {
                const fwhm = row.original?.shape?.fwhm;
                if (fwhm) {
                    return formatNumber(fwhm, peaksPreferences.fwhm.format);
                }
                return '';
            },
        },
        {
            showWhen: 'mu.show',
            index: 9,
            Header: 'mu',
            accessor: (row) => (row?.shape?.kind === 'pseudoVoigt' && row?.shape?.mu) || '',
            Cell: ({ row }) => {
                const mu = row.original?.shape?.kind === 'pseudoVoigt' &&
                    row.original?.shape?.mu;
                if (mu) {
                    return formatNumber(mu, peaksPreferences.mu.format);
                }
                return '';
            },
        },
        {
            showWhen: 'showEditPeakShapeAction',
            ...createActionColumn({
                index: 20,
                icon: _jsx(FaEdit, {}),
                onClick: editPeakHandler,
                style: {
                    borderRight: '0px',
                },
            }),
        },
        {
            showWhen: 'showDeleteAction',
            ...createActionColumn({
                index: 21,
                icon: _jsx(FaRegTrashAlt, {}),
                onClick: deletePeakHandler,
            }),
        },
    ], [
        peaksPreferences,
        saveDeltaPPMRefsHandler,
        deletePeakHandler,
        editPeakHandler,
    ]);
    const tableColumns = useMemo(() => {
        const columns = [];
        for (const col of COLUMNS) {
            const { showWhen, ...colParams } = col;
            if (lodashGet(peaksPreferences, showWhen)) {
                addCustomColumn(columns, colParams);
            }
        }
        return columns.sort((object1, object2) => object1.index - object2.index);
    }, [COLUMNS, peaksPreferences]);
    if (info?.isFid) {
        return _jsx(NoDataForFid, {});
    }
    if (!data || data.length === 0) {
        return _jsx(NoTableData, {});
    }
    return (_jsx(ReactTable, { activeRow: handleActiveRow, rowStyle: { activated: { backgroundColor: '#f5f5dc' } }, data: data, columns: tableColumns, approxItemHeight: 24, enableVirtualScroll: true }));
}
export default memo(PeaksTable);
