import { jsx as _jsx, Fragment as _Fragment, jsxs as _jsxs } from "react/jsx-runtime";
import { saveAs } from 'file-saver';
import lodashGet from 'lodash/get';
import { useMemo, useCallback, useState } from 'react';
import { FaCopy, FaRegTrashAlt, FaFileExport } from 'react-icons/fa';
import { IoColorPaletteOutline } from 'react-icons/io5';
import { isSpectrum1D } from '../../../data/data1d/Spectrum1D';
import { ClipboardFallbackModal } from '../../../utils/clipboard/clipboardComponents';
import { useClipboard } from '../../../utils/clipboard/clipboardHooks';
import { useDispatch } from '../../context/DispatchContext';
import { ContextMenu, } from '../../elements/ContextMenuBluePrint';
import ReactTable from '../../elements/ReactTable/ReactTable';
import { useAlert } from '../../elements/popup/Alert';
import { usePanelPreferences } from '../../hooks/usePanelPreferences';
import ExportAsJcampModal from '../../modal/ExportAsJcampModal';
import ColorIndicator from './base/ColorIndicator';
import { RenderAsHTML } from './base/RenderAsHTML';
import ShowHideSpectrumButton from './base/ShowHideSpectrumButton';
import { SpectrumName } from './base/SpectrumName';
function getActiveSpectraAsObject(activeSpectra) {
    const result = {};
    if (activeSpectra) {
        for (const activeSpectrum of activeSpectra) {
            if (activeSpectrum?.selected) {
                result[activeSpectrum.id] = true;
            }
        }
    }
    return result;
}
const columnStyle = {
    maxWidth: 0,
    overflow: 'hidden',
    height: '24px',
};
const options = [
    {
        text: 'Recolor based on distinct value',
        icon: _jsx(IoColorPaletteOutline, {}),
    },
];
var SpectraContextMenuOptionsKeys;
(function (SpectraContextMenuOptionsKeys) {
    SpectraContextMenuOptionsKeys["CopyToClipboard"] = "CopyToClipboard";
    SpectraContextMenuOptionsKeys["Delete"] = "Delete";
    SpectraContextMenuOptionsKeys["ExportAsJcamp"] = "ExportAsJcamp";
    SpectraContextMenuOptionsKeys["ExportAsText"] = "ExportAsText";
    SpectraContextMenuOptionsKeys["CopyAsText"] = "CopyAsText";
})(SpectraContextMenuOptionsKeys || (SpectraContextMenuOptionsKeys = {}));
const Spectra2DContextMenuOptions = [
    {
        text: 'Copy to Clipboard',
        icon: _jsx(FaCopy, {}),
        data: { id: SpectraContextMenuOptionsKeys.CopyToClipboard },
    },
    {
        text: 'Delete',
        icon: _jsx(FaRegTrashAlt, {}),
        data: { id: SpectraContextMenuOptionsKeys.Delete },
    },
];
const Spectra1DContextMenuOptions = [
    ...Spectra2DContextMenuOptions,
    {
        text: 'Export as JCAMP-DX',
        icon: _jsx(FaFileExport, {}),
        data: { id: SpectraContextMenuOptionsKeys.ExportAsJcamp },
    },
    {
        text: 'Export as text',
        icon: _jsx(FaFileExport, {}),
        data: { id: SpectraContextMenuOptionsKeys.ExportAsText },
    },
    {
        text: 'Copy as text',
        icon: _jsx(FaCopy, {}),
        data: { id: SpectraContextMenuOptionsKeys.CopyAsText },
    },
];
export function SpectraTable(props) {
    const { data, activeSpectra, onChangeVisibility, onOpenSettingModal, onChangeActiveSpectrum, nucleus, } = props;
    const alert = useAlert();
    const dispatch = useDispatch();
    const spectraPreferences = usePanelPreferences('spectra', nucleus);
    const activeSpectraObj = getActiveSpectraAsObject(activeSpectra);
    const [exportedSpectrum, setExportedSpectrum] = useState();
    const { rawWriteWithType, shouldFallback, cleanShouldFallback, text } = useClipboard();
    const COLUMNS = useMemo(() => ({
        visible: {
            id: 'hide-show-spectrum',
            Header: '',
            style: {
                width: '35px',
                maxWidth: '55px',
                height: '24px',
            },
            Cell: ({ row }) => {
                return (_jsx(ShowHideSpectrumButton, { data: row.original, onChangeVisibility: onChangeVisibility }));
            },
        },
        color: {
            id: 'spectrum-actions',
            style: {
                width: '30px',
                maxWidth: '30px',
                height: '24px',
            },
            Cell: ({ row }) => {
                const { display, info: { dimension, isFid }, } = row.original;
                if (dimension === 2 && isFid) {
                    return _jsx("div", {});
                }
                return (_jsx(ColorIndicator, { display: display, dimension: dimension, onClick: (event) => onOpenSettingModal(event, row.original) }));
            },
        },
    }), [onChangeVisibility, onOpenSettingModal]);
    const selectContextMenuHandler = useCallback((option, spectrum) => {
        const { id } = option;
        switch (id) {
            case SpectraContextMenuOptionsKeys.CopyToClipboard: {
                void (async () => {
                    const { data, info } = spectrum;
                    await rawWriteWithType(JSON.stringify({ data, info }, (_, value) => ArrayBuffer.isView(value) ? Array.from(value) : value, 2));
                    alert.success('Data copied to clipboard');
                })();
                break;
            }
            case SpectraContextMenuOptionsKeys.Delete: {
                setTimeout(() => {
                    dispatch({ type: 'DELETE_SPECTRA', payload: { id: spectrum.id } });
                }, 0);
                break;
            }
            case SpectraContextMenuOptionsKeys.ExportAsJcamp: {
                setExportedSpectrum(spectrum);
                break;
            }
            case SpectraContextMenuOptionsKeys.ExportAsText: {
                const data = convertSpectrumToText(spectrum);
                const blob = new Blob([data], { type: 'text/plain' });
                saveAs(blob, `${spectrum.info.name}.tsv`);
                break;
            }
            case SpectraContextMenuOptionsKeys.CopyAsText: {
                const data = convertSpectrumToText(spectrum);
                void rawWriteWithType(data, 'text/plain').then(() => alert.success('Spectrum copied to clipboard'));
                break;
            }
            default: {
                break;
            }
        }
    }, [alert, dispatch, rawWriteWithType]);
    function handleActiveRow(row) {
        return activeSpectraObj?.[row?.original.id] || false;
    }
    const tableColumns = useMemo(() => {
        const columns = [];
        let index = 0;
        const visibleColumns = spectraPreferences.columns.filter((col) => col.visible);
        for (const col of visibleColumns) {
            const name = col?.name;
            const path = col?.jpath;
            if (name && COLUMNS[name]) {
                columns.push({
                    ...COLUMNS[name],
                    Header: () => _jsx(ColumnHeader, { label: col.label, col: col }),
                    id: name,
                });
            }
            else {
                const pathString = Array.isArray(path) ? path.join('.') : '';
                let style = columnStyle;
                let cellRender = null;
                if (pathString === 'info.name') {
                    if (visibleColumns.length > 3) {
                        style = { ...columnStyle, width: '50%' };
                    }
                    cellRender = ({ row }) => {
                        return _jsx(SpectrumName, { data: row.original });
                    };
                }
                if (pathString === 'info.solvent') {
                    cellRender = ({ row }) => {
                        return _jsx(RenderAsHTML, { data: row.original, jpath: pathString });
                    };
                }
                const cell = {
                    Header: () => _jsx(ColumnHeader, { label: col.label, col: col }),
                    accessor: (row) => lodashGet(row, path, ''),
                    ...(cellRender && { Cell: cellRender }),
                    id: `${index}`,
                    style,
                };
                columns.push(cell);
            }
            index++;
        }
        return columns;
    }, [COLUMNS, spectraPreferences.columns]);
    function handleSortEnd(data) {
        dispatch({
            type: 'ORDER_SPECTRA',
            payload: {
                data,
            },
        });
    }
    function handleRowStyle(data) {
        return {
            base: activeSpectraObj?.[data?.original.id] ? { opacity: 0.2 } : {},
            activated: { opacity: 1 },
        };
    }
    const contextMenu = nucleus.split(',').length === 1
        ? Spectra1DContextMenuOptions
        : Spectra2DContextMenuOptions;
    return (_jsxs(_Fragment, { children: [_jsx(ReactTable, { rowStyle: handleRowStyle, activeRow: handleActiveRow, data: data, columns: tableColumns, onClick: (e, data) => onChangeActiveSpectrum(e, data.original), enableVirtualScroll: true, approxItemHeight: 24, contextMenu: contextMenu, onContextMenuSelect: selectContextMenuHandler, onSortEnd: handleSortEnd, style: { 'table td': { paddingTop: 0, paddingBottom: 0 } } }), exportedSpectrum && (_jsx(ExportAsJcampModal, { spectrum: exportedSpectrum, closeDialog: () => {
                    setExportedSpectrum(null);
                } })), _jsx(ClipboardFallbackModal, { mode: shouldFallback, onDismiss: cleanShouldFallback, text: text, label: "Spectra" })] }));
}
const ColumnHeader = ({ label, col, }) => {
    const dispatch = useDispatch();
    function selectHandler() {
        if (col?.jpath) {
            dispatch({
                type: 'RECOLOR_SPECTRA_COLOR',
                payload: { jpath: col?.jpath },
            });
        }
    }
    return (_jsx(ContextMenu, { options: options, onSelect: selectHandler, style: {
            whiteSpace: 'nowrap',
            overflow: 'hidden',
            textOverflow: 'ellipsis',
        }, children: label }));
};
function convertSpectrumToText(spectrum) {
    if (!isSpectrum1D(spectrum))
        return '';
    const { data: { x, re }, } = spectrum;
    const lines = ['x\ty'];
    for (let i = 0; i < x.length; i++) {
        lines.push(`${x[i]}\t${re[i]}`);
    }
    return lines.join('\n');
}
