import { jsx as _jsx, Fragment as _Fragment, jsxs as _jsxs } from "@emotion/react/jsx-runtime";
/** @jsxImportSource @emotion/react */
import { css } from '@emotion/react';
import { readFromWebSource, serializeNmriumState, } from 'nmr-load-save';
import { mapRanges } from 'nmr-processing';
import OCL from 'openchemlib/full';
import { useCallback, useState, useRef, memo, useEffect, useMemo } from 'react';
import { useAccordionContext, useOnOff } from 'react-science/ui';
import { isSpectrum1D } from '../../../data/data1d/Spectrum1D';
import { getSum } from '../../../data/data1d/Spectrum1D/SumManager';
import { initiateDatabase, prepareData, DATA_BASES, } from '../../../data/data1d/database';
import { useChartData } from '../../context/ChartContext';
import { useDispatch } from '../../context/DispatchContext';
import { usePreferences } from '../../context/PreferencesContext';
import { useAlert } from '../../elements/popup/Alert';
import { useFormatNumberByNucleus } from '../../hooks/useFormatNumberByNucleus';
import { options } from '../../toolbar/ToolTypes';
import Events from '../../utility/Events';
import { exportAsJSON } from '../../utility/export';
import nucleusToString from '../../utility/nucleusToString';
import { PanelNoData } from '../PanelNoData';
import { tablePanelStyle } from '../extra/BasicPanelStyle';
import NoTableData from '../extra/placeholder/NoTableData';
import PreferencesHeader from '../header/PreferencesHeader';
import DatabasePreferences from './DatabasePreferences';
import { DatabaseSearchOptions } from './DatabaseSearchOptions';
import { DatabaseStructureSearchModal } from './DatabaseStructureSearchModal';
import DatabaseTable from './DatabaseTable';
const emptyKeywords = {
    solvent: '-1',
    searchKeywords: '',
};
function mapKeywordsToArray(searchKeywords, solvent) {
    const values = searchKeywords
        ? searchKeywords
            .trim()
            .split(' ')
            .filter((value) => value !== '')
        : [];
    if (solvent !== '-1') {
        values.unshift(`solvent:${solvent}`);
    }
    return values;
}
function DatabasePanelInner({ nucleus, selectedTool, databases, defaultDatabase, }) {
    const dispatch = useDispatch();
    const alert = useAlert();
    const { item } = useAccordionContext('Databases');
    const format = useFormatNumberByNucleus(nucleus);
    const [isFlipped, setFlipStatus] = useState(false);
    const [isOpenSearchByStructure, openSearchByStructure, closeSearchByStructure,] = useOnOff(false);
    const settingRef = useRef();
    const [keywords, setKeywords] = useState(emptyKeywords);
    const databaseInstance = useRef(null);
    const databaseDataRef = useRef([]);
    const [result, setResult] = useState({
        data: [],
        databases: [],
        solvents: [],
    });
    const [idCode, setIdCode] = useState();
    function settingsPanelHandler() {
        setFlipStatus((flag) => !flag);
    }
    function saveSettingHandler() {
        settingRef.current.saveSetting();
        setFlipStatus(false);
    }
    const search = useCallback((solvents) => {
        const { solvent, searchKeywords } = keywords;
        if (databaseInstance.current) {
            const keywords = mapKeywordsToArray(searchKeywords, solvent);
            const data = databaseInstance.current.search({ keywords, idCode });
            setResult((prevResult) => ({
                ...prevResult,
                data,
                ...(solvents && { solvents }),
            }));
        }
    }, [idCode, keywords]);
    useEffect(() => {
        const { solvent, searchKeywords } = keywords;
        setTimeout(async () => {
            if (databaseInstance.current) {
                const hideLoading = await alert.showLoading(`Preparing of the Result`);
                if (solvent === '-1' && !searchKeywords) {
                    const solvents = mapSolventsToSelect(databaseInstance.current.getSolvents());
                    search(solvents);
                }
                else {
                    search();
                }
                hideLoading();
            }
        }, 0);
    }, [alert, idCode, keywords, search]);
    useEffect(() => {
        function handle(event) {
            if (selectedTool === options.databaseRangesSelection.id) {
                setKeywords((prevState) => {
                    const oldKeywords = prevState.searchKeywords
                        ? prevState.searchKeywords.split(' ')
                        : [];
                    const [from, to] = event.range;
                    const searchKeywords = [
                        ...oldKeywords,
                        `delta:${format(from)}..${format(to)}`,
                    ].join(' ');
                    return { ...prevState, searchKeywords };
                });
            }
        }
        Events.on('brushEnd', handle);
        return () => {
            Events.off('brushEnd', handle);
        };
    }, [format, selectedTool]);
    useEffect(() => {
        setTimeout(async () => {
            if (databaseInstance.current) {
                const hideLoading = await alert.showLoading(`Loading the database`);
                databaseInstance.current = initiateDatabase(databaseDataRef.current, nucleus);
                hideLoading();
                setKeywords({ ...emptyKeywords });
            }
        }, 0);
    }, [alert, nucleus]);
    const handleChangeDatabase = useCallback((databaseKey) => {
        const database = databases.find((item) => item.key === databaseKey);
        setTimeout(async () => {
            if (database?.url) {
                const { url, label } = database;
                const hideLoading = await alert.showLoading(`load ${label} database`);
                try {
                    databaseDataRef.current = await fetch(url)
                        .then((response) => response.json())
                        .then((databaseRecords) => databaseRecords.map((record) => ({
                        ...record,
                        baseURL: url,
                    })));
                }
                catch {
                    alert.error(`Failed to load ${url}`);
                }
                finally {
                    hideLoading();
                }
            }
            else {
                databaseDataRef.current = database
                    ?.value;
            }
            const hideLoading = await alert.showLoading(`Loading the database`);
            databaseInstance.current = initiateDatabase(databaseDataRef.current, nucleus);
            setKeywords({ ...emptyKeywords });
            hideLoading();
        }, 0);
    }, [alert, databases, nucleus]);
    useEffect(() => {
        if (item?.isOpen && defaultDatabase && !databaseInstance.current) {
            handleChangeDatabase(defaultDatabase);
        }
    }, [databases, defaultDatabase, handleChangeDatabase, item?.isOpen, nucleus]);
    const tableData = useMemo(() => {
        return prepareData(result.data);
    }, [result.data]);
    const resurrectHandler = useCallback((rowData) => {
        const { index, baseURL, jcampURL: jcampRelativeURL } = rowData;
        const { ranges, solvent, names = [] } = result.data[index];
        if (jcampRelativeURL) {
            const url = new URL(jcampRelativeURL, baseURL);
            setTimeout(async () => {
                const hideLoading = await alert.showLoading(`load jcamp in progress...`);
                try {
                    const { data } = await readFromWebSource({
                        entries: [{ baseURL: url.origin, relativePath: url.pathname }],
                    });
                    const spectrum = data?.spectra?.[0] || null;
                    if (spectrum && isSpectrum1D(spectrum)) {
                        dispatch({
                            type: 'RESURRECTING_SPECTRUM_FROM_JCAMP',
                            payload: { ranges, spectrum },
                        });
                    }
                }
                catch {
                    alert.error(`Failed to load Jcamp`);
                }
                finally {
                    hideLoading();
                }
            }, 0);
        }
        else {
            dispatch({
                type: 'RESURRECTING_SPECTRUM_FROM_RANGES',
                payload: { ranges, info: { solvent, nucleus, name: names[0] } },
            });
        }
    }, [alert, dispatch, nucleus, result.data]);
    const saveHandler = useCallback((row) => {
        if (row?.jcampURL) {
            setTimeout(async () => {
                const hideLoading = await alert.showLoading(`Download jcamp in progress...`);
                try {
                    await saveJcampAsJson(row, result);
                    hideLoading();
                }
                catch {
                    alert.error(`Failed to download the jcamp`);
                }
                finally {
                    hideLoading();
                }
            }, 0);
        }
        else {
            alert.error(`No jcamp file to save`);
        }
    }, [alert, result]);
    const searchByStructureHandler = (idCodeValue) => {
        setIdCode(idCodeValue);
    };
    return (_jsxs("div", { css: [
            tablePanelStyle,
            isFlipped &&
                css `
            .table-container {
              table,
              th {
                position: relative !important;
              }
            }
          `,
        ], children: [!isFlipped && (_jsxs(_Fragment, { children: [_jsx(DatabaseSearchOptions, { databases: databases, defaultDatabase: defaultDatabase, idCode: idCode, keywords: keywords, result: result, total: databaseInstance.current?.data.length || 0, onKeywordsChange: (options) => setKeywords((prevKeywords) => ({ ...prevKeywords, ...options })), onSettingClick: settingsPanelHandler, onStructureClick: openSearchByStructure, onDatabaseChange: handleChangeDatabase }), _jsx(DatabaseStructureSearchModal, { isOpen: isOpenSearchByStructure, onClose: closeSearchByStructure, onChange: searchByStructureHandler, idCode: idCode })] })), isFlipped && (_jsx(PreferencesHeader, { onSave: saveSettingHandler, onClose: settingsPanelHandler })), _jsx("div", { className: "inner-container", children: !isFlipped ? (tableData && tableData.length > 0 ? (_jsx(DatabaseTable, { data: tableData, totalCount: result.data.length, onAdd: resurrectHandler, onSave: saveHandler })) : (_jsx(NoTableData, { text: databases && databases?.length > 0
                        ? 'Please select a database'
                        : 'Please add databases URL in the general preferences' }))) : (_jsx(DatabasePreferences, { ref: settingRef })) })] }));
}
const MemoizedDatabasePanel = memo(DatabasePanelInner);
export default function PeaksPanel() {
    const { view: { spectra: { activeTab }, }, toolOptions: { selectedTool }, displayerMode, } = useChartData();
    const { current } = usePreferences();
    const { data, defaultDatabase } = current.databases;
    const databases = DATA_BASES.concat(data.filter((datum) => datum.enabled));
    if (!activeTab || displayerMode !== '1D') {
        return (_jsx(PanelNoData, { children: "Databases are only available when 1D experimental spectrum is displayed. It will automatically select the corresponding nucleus" }));
    }
    return (_jsx(MemoizedDatabasePanel, { nucleus: activeTab, selectedTool: selectedTool, databases: databases, defaultDatabase: defaultDatabase }));
}
function mapSolventsToSelect(solvents) {
    const result = solvents.map((key) => {
        return {
            label: key,
            value: key,
        };
    });
    result.unshift({ label: 'All', value: '-1' });
    return result;
}
async function saveJcampAsJson(rowData, filteredData) {
    const { index, baseURL, jcampURL, names, ocl = {}, smiles } = rowData;
    const { ranges } = filteredData.data[index];
    const url = new URL(jcampURL, baseURL);
    const { data: { spectra, source } = { source: {}, spectra: [] }, version } = await readFromWebSource({
        entries: [{ baseURL: url.origin, relativePath: url.pathname }],
    });
    let molfile = '';
    let molecule = null;
    if (ocl?.idCode) {
        molecule = OCL.Molecule.fromIDCode(ocl.idCode);
        molfile = molecule.toMolfileV3();
    }
    else if (smiles) {
        molecule = OCL.Molecule.fromSmiles(smiles);
        molfile = molecule.toMolfileV3();
    }
    const spectraData = [];
    for (const spectrum of spectra) {
        if (spectrum.info.dimension === 1) {
            let sum = 0;
            if (molecule) {
                sum = getSum(molecule.getMolecularFormula().formula, nucleusToString(spectrum.info.nucleus));
            }
            spectraData.push({
                ...spectrum,
                ranges: {
                    options: { sum: sum || 100 },
                    values: mapRanges(ranges, spectrum),
                },
            });
        }
    }
    const exportedData = serializeNmriumState({
        version,
        data: {
            source,
            spectra: spectraData,
            ...(molfile && { molecules: [{ molfile }] }),
        },
    }, { includeData: 'dataSource' });
    await exportAsJSON(exportedData, names?.[0], 1);
}
