import { jsx as _jsx, jsxs as _jsxs, Fragment as _Fragment } from "@emotion/react/jsx-runtime";
/** @jsxImportSource @emotion/react */
import { css } from '@emotion/react';
import { Formik } from 'formik';
import { xFindClosestIndex } from 'ml-spectra-processing';
import { useEffect, useState } from 'react';
import * as Yup from 'yup';
import { REFERENCES } from '../../../data/constants/References';
import { useDispatch } from '../../context/DispatchContext';
import Button from '../../elements/Button';
import Label from '../../elements/Label';
import Message from '../../elements/Message';
import Select from '../../elements/Select';
import FormikInput from '../../elements/formik/FormikInput';
import useSpectraByActiveNucleus from '../../hooks/useSpectraPerNucleus';
import Events from '../../utility/Events';
const labelStyle = {
    label: { flex: 4, fontWeight: '500' },
    wrapper: { flex: 8, display: 'flex', alignItems: 'center' },
    container: { padding: '5px 0' },
};
const inputStyle = {
    input: {
        padding: '5px',
    },
};
const baseList = [{ key: 1, value: 'manual', label: 'Manual' }];
const DEFAULT_OPTIONS = {
    from: -1,
    to: 1,
    nbPeaks: 1,
    targetX: 0,
};
const schemaValidation = Yup.object({
    from: Yup.number().required(),
    to: Yup.number().required(),
    nbPeaks: Yup.number().required(),
    targetX: Yup.number().required(),
});
function checkSpectra(options, spectra) {
    const { from, to } = options;
    for (const spectrum of spectra) {
        const { data: { x }, } = spectrum;
        const min = x[0];
        const max = x.at(-1);
        if (from < min || to > max) {
            throw new Error('Some spectra do not have data in the selected range');
        }
        if (Math.abs(xFindClosestIndex(x, from) - xFindClosestIndex(x, to)) < 10) {
            throw new Error('The selected range is too small to provide accurate results');
        }
    }
}
function checkOptions(options) {
    const returnedOptions = { ...options };
    if (options.from > options.to) {
        returnedOptions.to = options.from;
        returnedOptions.from = options.to;
    }
    return returnedOptions;
}
function getList(nucleus) {
    if (!REFERENCES?.[nucleus]) {
        return [];
    }
    const list = Object.entries(REFERENCES[nucleus]).map((item) => ({
        value: item[0],
        label: item[0],
    }));
    return baseList.concat(list);
}
function AlignSpectra({ onClose = () => null, nucleus }) {
    const spectra = useSpectraByActiveNucleus();
    const dispatch = useDispatch();
    const [options, setOptions] = useState(DEFAULT_OPTIONS);
    const [error, setError] = useState('');
    function submitHandler(inputOptions) {
        const options = checkOptions(inputOptions);
        setOptions(options);
        try {
            checkSpectra(options, spectra);
            dispatch({ type: 'ALIGN_SPECTRA', payload: options });
            onClose();
        }
        catch (error) {
            const message = error.message;
            setError(message);
        }
    }
    useEffect(() => {
        function handler(event) {
            const [from, to] = event.range;
            setOptions((prevOptions) => ({ ...prevOptions, from, to }));
        }
        Events.on('brushEnd', handler);
        return () => {
            Events.off('brushEnd', handler);
        };
    }, []);
    function optionChangeHandler(key) {
        const { delta: targetX = 0, ...otherOptions } = REFERENCES?.[nucleus]?.[key] || {};
        const value = {
            ...DEFAULT_OPTIONS,
            targetX,
            ...otherOptions,
        };
        setOptions(value);
        setError('');
    }
    const List = getList(nucleus);
    const styles = css `
    max-height: 100%;
    padding: 10px 0 5px 20px;
    display: flex;
    flex-direction: column;
    flex: 1;

    .body {
      overflow: auto;
      padding: 10px 10px 25px 0;
    }

    .header {
      padding: 5px 0;
      font-size: 15px;
      font-weight: bold;
    }

    .footer {
      display: flex;
      padding-top: 5px;
    }
  `;
    return (_jsx("div", { css: styles, children: _jsx(Formik, { initialValues: options, enableReinitialize: true, onSubmit: submitHandler, validationSchema: schemaValidation, validate: () => setError(''), children: ({ submitForm }) => (_jsxs(_Fragment, { children: [_jsxs("div", { className: "body", style: { flex: 1 }, children: [_jsx("div", { className: "header", children: _jsx("span", { children: "Spectra calibration" }) }), error && _jsx(Message, { type: "error", children: error }), _jsx(Label, { title: "Options", style: labelStyle, children: _jsx(Select, { items: List, style: { width: '100%', height: 30 }, onChange: optionChangeHandler }) }), _jsxs(Label, { title: "Range", style: labelStyle, children: [_jsx(Label, { title: "From", children: _jsx(FormikInput, { name: "from", type: "number", style: inputStyle }) }), _jsx(Label, { title: "To", style: { label: { padding: '0 10px' } }, children: _jsx(FormikInput, { name: "to", type: "number", style: inputStyle }) })] }), _jsx(Label, { title: "Number of Peaks", style: labelStyle, children: _jsx(FormikInput, { name: "nbPeaks", type: "number", style: inputStyle }) }), _jsx(Label, { title: "Target PPM", style: labelStyle, children: _jsx(FormikInput, { name: "targetX", type: "number", style: inputStyle }) })] }), _jsx("div", { className: "footer", children: _jsx(Button.Done, { style: { padding: '5px 14px', fontSize: 14 }, onClick: submitForm, children: "Done" }) })] })) }) }));
}
export default AlignSpectra;
