import { jsx as _jsx, jsxs as _jsxs } from "react/jsx-runtime";
import { css } from '@emotion/react';
import lodashGet from 'lodash/get';
import { useMemo, useRef, useState } from 'react';
import { ResponsiveChart } from 'react-d3-utils';
import { FaFileExport } from 'react-icons/fa';
import { Axis, LineSeries, Plot } from 'react-plot';
import { useChartData } from '../../context/ChartContext';
import Button from '../../elements/Button';
import Input from '../../elements/Input';
import Label from '../../elements/Label';
import { useAlert } from '../../elements/popup/Alert';
import useSpectraByActiveNucleus from '../../hooks/useSpectraPerNucleus';
import { copyPNGToClipboard } from '../../utility/export';
import { getSpectraObjectPaths } from '../../utility/getSpectraObjectPaths';
const padding = 15;
const svgId = 'multiple-analysis-svg';
function prepareAnalysisData(analysisData, columnKey) {
    const valueKey = analysisData.options.columns?.[columnKey]?.valueKey;
    const spectraAnalysis = {};
    if (valueKey) {
        for (const record of analysisData.values) {
            const data = record?.[columnKey];
            spectraAnalysis[data.SID] = data[valueKey];
        }
    }
    return spectraAnalysis;
}
function usePlotData(analysisData, options) {
    const spectra = useSpectraByActiveNucleus();
    const { yPath, xPath, paths } = options;
    const xPathKeys = paths?.[xPath];
    const yPathKeys = paths?.[yPath];
    let xData;
    let yData;
    const analysisColumns = getAnalysisColumns(analysisData);
    if (analysisColumns.includes(xPath)) {
        xData = prepareAnalysisData(analysisData, xPath);
    }
    if (analysisColumns.includes(yPath)) {
        yData = prepareAnalysisData(analysisData, yPath);
    }
    return spectra.map((spectrum, index) => {
        const x = xData
            ? xData[spectrum.id]
            : lodashGet(spectrum, xPathKeys, index);
        const y = yData
            ? yData[spectrum.id]
            : lodashGet(spectrum, yPathKeys, index);
        return {
            x,
            y,
        };
    });
}
function getAnalysisColumns(spectraAnalysisData) {
    return Object.keys(spectraAnalysisData.options.columns);
}
function getAnalysisColumnsPaths(spectraAnalysisData) {
    const datalist = getAnalysisColumns(spectraAnalysisData);
    const paths = {};
    for (const key of datalist) {
        paths[key] = [key];
    }
    return {
        datalist,
        paths,
    };
}
export default function AnalysisChart(props) {
    const { spectraAnalysisData } = props;
    const { data } = useChartData();
    const alert = useAlert();
    const chartParentRef = useRef(null);
    const [plotOptions, setPlotOptions] = useState({
        xPath: '',
        yPath: '',
    });
    const { datalist, paths } = useMemo(() => {
        const { datalist: xDataList, paths: xPaths } = getSpectraObjectPaths(data, [
            'info',
            'meta',
            'customInfo',
        ]);
        const { datalist: yDataList, paths: yPaths } = getAnalysisColumnsPaths(spectraAnalysisData);
        const datalist = yDataList.concat(xDataList);
        const paths = { ...yPaths, ...xPaths };
        return { datalist, paths };
    }, [data, spectraAnalysisData]);
    function handleChangeKey(event) {
        const { name, value } = event.target;
        setPlotOptions((prevOptions) => ({ ...prevOptions, [name]: value }));
    }
    function handleCopy() {
        if (chartParentRef.current) {
            void copyPNGToClipboard(chartParentRef.current, svgId, css({ text: { fill: 'black' } }));
            alert.success('Chart copied to clipboard');
        }
    }
    const plotData = usePlotData(spectraAnalysisData, { ...plotOptions, paths });
    const xLabel = paths?.[plotOptions.xPath]?.at(-1) || '';
    const yLabel = paths?.[plotOptions.yPath]?.at(-1) || '';
    return (_jsxs("div", { children: [_jsxs("div", { style: { display: 'flex', padding: '5px' }, children: [_jsx(Label, { title: "X ", children: _jsx(Input, { name: "xPath", style: {
                                input: { padding: '5px' },
                            }, value: plotOptions.xPath, datalist: datalist, onChange: handleChangeKey }) }), _jsx(Label, { title: "Y ", style: { container: { paddingLeft: '5px' } }, children: _jsx(Input, { name: "yPath", style: {
                                input: { padding: '5px' },
                            }, value: plotOptions.yPath, datalist: datalist, onChange: handleChangeKey }) }), _jsx(Button.BarButton, { toolTip: "Copy chart to clipboard", onClick: handleCopy, children: _jsx(FaFileExport, {}) })] }), _jsx("div", { style: {
                    width: '100%',
                    position: 'relative',
                    height: '220px',
                }, ref: chartParentRef, children: _jsx(ResponsiveChart, { children: ({ width, height }) => (_jsxs(Plot, { width: width, height: height, svgClassName: svgId, svgId: svgId, margin: {
                            top: padding,
                            right: padding,
                            left: padding,
                            bottom: padding,
                        }, children: [_jsx(LineSeries, { data: plotData, xAxis: "x", yAxis: "y" }), _jsx(Axis, { id: "x", label: `X ( ${xLabel} )`, position: "bottom", tickLabelStyle: { fontSize: '0.6rem' }, labelStyle: { fontSize: '0.6rem' } }), _jsx(Axis, { id: "y", label: `Y ( ${yLabel} )`, position: "left", tickLabelStyle: { fontSize: '0.6rem' }, labelStyle: { fontSize: '0.7rem' } })] })) }) })] }));
}
