import { produce } from 'immer';
import { getLocalStorage, removeData, storeData, } from '../../utility/LocalStorage';
import Workspaces from '../../workspaces';
import { addWorkspace } from './actions/addWorkspace';
import { analyzeSpectra, changeAnalysisColumnValueKey, deleteAnalysisColumn, setSpectraAnalysisPanelsPreferences, } from './actions/analyzeSpectra';
import { applyGeneralPreferences } from './actions/applyGeneralPreferences';
import { initPreferences } from './actions/initPreferences';
import { setMatrixGenerationOptions, addExclusionZone, deleteExclusionZone, } from './actions/matrixGeneration';
import { removeWorkspace } from './actions/removeWorkspace';
import { setActiveWorkspace } from './actions/setActiveWorkspace';
import { setPanelsPreferences } from './actions/setPanelsPreferences';
import { setPreferences } from './actions/setPreferences';
import { setVerticalSplitterPosition } from './actions/setVerticalSplitterPosition';
import { setWorkspace } from './actions/setWorkspace';
import { mapWorkspaces } from './utilities/mapWorkspaces';
const LOCAL_STORAGE_VERSION = 17;
export const WORKSPACES_KEYS = {
    componentKey: `nmrium-component-workspace`,
    nmriumKey: `nmrium-file-workspace`,
};
export const WORKSPACES = [
    {
        key: 'default',
        label: Workspaces.default.label,
    },
    {
        key: 'process1D',
        label: Workspaces.process1D.label,
    },
    {
        key: 'exercise',
        label: Workspaces.exercise.label,
    },
    {
        key: 'prediction',
        label: Workspaces.prediction.label,
    },
    {
        key: 'assignment',
        label: Workspaces.assignment.label,
    },
    {
        key: 'embedded',
        label: Workspaces.embedded.label,
    },
    {
        key: 'simulation',
        label: Workspaces.simulation.label,
    },
];
export const preferencesInitialState = {
    version: LOCAL_STORAGE_VERSION,
    workspaces: {},
    originalWorkspaces: {},
    dispatch: () => null,
    workspace: {
        current: 'default',
        base: null,
    },
};
export function initPreferencesState(state) {
    const nmrLocalStorageVersion = getLocalStorage('nmr-local-storage-version', false);
    const localData = getLocalStorage('nmr-general-settings');
    // remove old nmr-local-storage-version key
    if (nmrLocalStorageVersion && localData?.version) {
        removeData('nmr-local-storage-version');
    }
    //  if the local setting version != current settings version number
    if (!localData?.version || localData?.version !== LOCAL_STORAGE_VERSION) {
        removeData('nmr-general-settings');
        const data = {
            version: LOCAL_STORAGE_VERSION,
            workspaces: {},
        };
        storeData('nmr-general-settings', JSON.stringify(data));
    }
    const predefinedWorkspaces = mapWorkspaces(Workspaces, {
        source: 'predefined',
    });
    const localWorkspaces = mapWorkspaces(localData?.workspaces || {}, {
        source: 'user',
    });
    return {
        ...state,
        originalWorkspaces: { ...predefinedWorkspaces, ...localWorkspaces },
        workspaces: { ...predefinedWorkspaces, ...localWorkspaces },
        workspace: {
            current: localData?.currentWorkspace || 'default',
            base: null,
        },
    };
}
function innerPreferencesReducer(draft, action) {
    switch (action.type) {
        case 'INIT_PREFERENCES':
            return initPreferences(draft, action);
        case 'SET_PREFERENCES':
            return setPreferences(draft, action);
        case 'SET_PANELS_PREFERENCES':
            return setPanelsPreferences(draft, action);
        case 'SET_WORKSPACE':
            return setWorkspace(draft, action);
        case 'SET_ACTIVE_WORKSPACE':
            return setActiveWorkspace(draft, action);
        case 'ADD_WORKSPACE':
            return addWorkspace(draft, action);
        case 'REMOVE_WORKSPACE':
            return removeWorkspace(draft, action);
        case 'APPLY_General_PREFERENCES':
            return applyGeneralPreferences(draft, action);
        case 'ANALYZE_SPECTRA':
            return analyzeSpectra(draft, action);
        case 'CHANGE_ANALYSIS_COLUMN_VALUE_KEY':
            return changeAnalysisColumnValueKey(draft, action);
        case 'DELETE_ANALYSIS_COLUMN':
            return deleteAnalysisColumn(draft, action);
        case 'SET_SPECTRA_ANALYSIS_PREFERENCES':
            return setSpectraAnalysisPanelsPreferences(draft, action);
        case 'SET_MATRIX_GENERATION_OPTIONS':
            return setMatrixGenerationOptions(draft, action);
        case 'ADD_MATRIX_GENERATION_EXCLUSION_ZONE':
            return addExclusionZone(draft, action);
        case 'DELETE_MATRIX_GENERATION_EXCLUSION_ZONE':
            return deleteExclusionZone(draft, action);
        case 'SET_VERTICAL_SPLITTER_POSITION':
            return setVerticalSplitterPosition(draft, action);
        default:
            return draft;
    }
}
const preferencesReducer = produce(innerPreferencesReducer);
export default preferencesReducer;
